#!/bin/bash

source /opt/velostrata/velostrata-functions

# Detect migration/bursting phase
# Output values:
#   origin
#   velos
#   detach
#   idle
#
function detect_phase() {
	local platform last_platform last_ibft

	platform=$1
	last_platform=$(get_last "platform")
	last_ibft=$(get_last "ibft")

	# process init environment
	eval "$(xargs -n 1 -0 < /proc/1/environ)"

	if [[ ! -z "${HC_VERSION}" ]] ; then
	 	echo hc
	elif [[ -e /sys/firmware/ibft ]] ; then
	 	echo velos
	elif [[ -z "${last_platform}" ]] && [[ "${platform}" != "gcp" ]] || [[ "${last_platform}" == "unknown" ]] ; then
		echo origin
	elif [[ "${platform}" == "${last_platform}" ]] ; then
		[[ "${last_ibft}" -eq 1 ]] && echo detach || echo idle
	else
		[[ "${last_ibft}" -eq 1 ]] && echo origin || echo detach
	fi
}


# Execute action(s)
# $1 - script to execute
#
function execute() {
	bash -c "$2"
	rc=$?
	if [ $rc -eq 0 ]; then
		velos_log "$1 - OK"
	else
		velos_log "$1 - FAILED"
	fi
}

# Bypass action(s)
# $1 - script to execute
#
function bypass() {
	velos_log "$1 - BYPASS"
}


# Process single rule
# $1 - env
# $2 - rule
# $3 - insertion point (default: velos service start)
#
function process_rule() {
	local env rule after parts key value NAME PLATFORM NEGATIVE_PLATFORM TEST NEGATIVE_TEST REQUIRES ACTION
	env=$1
	rule=$2
	after=$3

	if [[ "${rule}" == "" ]] || [[ "${rule}" == "#"* ]] ; then
		return
	fi

	parts=`echo ${rule} | awk -F '[,]' '{ for(i=1;i<=NF;i++) { print $i } }'`
	while read -r entry ; do
		key=$(echo ${entry} | cut -d = -f 1)
		value=$(echo ${entry} | cut -d = -f 2 | tr -d '"')
		if [ "${key}" == 'NAME' ] ; then NAME="${value}" ; fi
		if [ "${key}" == 'PLATFORM' ] ; then PLATFORM="${value}" ; fi
		if [ "${key}" == 'PLATFORM!' ] ; then NEGATIVE_PLATFORM="${value}" ; fi
		if [ "${key}" == 'TEST' ] ; then TEST="${value}" ; fi
		if [ "${key}" == 'TEST!' ] ; then NEGATIVE_TEST="${value}" ; fi
		if [ "${key}" == 'REQUIRES' ] ; then REQUIRES="${value}" ; fi
		if [ "${key}" == 'ACTION' ] ; then ACTION="${value}" ; fi
	done < <(echo "${parts}")

	if [ -z "${NAME}" ] ; then
		NAME="${ACTION}"
	fi

	if [[ ! -z "${PLATFORM}" ]] && [ "${PLATFORM}" != "${env}" ] && [ "${PLATFORM}" != "any" ] ; then
		bypass "${NAME}"
	elif [[ ! -z "${NEGATIVE_PLATFORM}" ]] && [ "${NEGATIVE_PLATFORM}" == "${env}" ] ; then
		bypass "${NAME}"
	elif [ "${TEST}" != "" ] && [ ! -e "${TEST}" ] ; then
		bypass "${NAME}"
	elif [ "${NEGATIVE_TEST}" != "" ] && [ -e "${NEGATIVE_TEST}" ] ; then
		bypass "${NAME}"
	elif ( [ "${after}" != "" ] || [ "${REQUIRES}" != "" ] ) && [ "${after}" != "${REQUIRES}" ] ; then
		bypass "${NAME}"
 	else
		execute "${NAME}" "${ACTION}"
	fi
}


# Process rules file
# $1 - env
# $2 - rules file
# $3 - insertion point (default: velos service start)
#
function process_rules() {
	local env file after
	env=$1
	file=$2
	after=$3

	if [ -e ${file} ]; then
		velos_log "Processing: ${file}"
		while read line; do
			process_rule "${env}" "${line}" "${after}"
		done <${file}
	fi
}


# Match and run actions based on conditions
# $1 - env
# $2 - phase
# $3 - insertion point (default: velos service start)
#
function match_and_run() {
	local phase env after
	phase=$1
	env=$2
	after=$3

	velos_log "Running actions for ${phase} on ${env}, after=${after}"
	process_rules "${env}" /opt/velostrata/actions/${phase}.rules "${after}"

	if [ "${after}" = "network" ] ; then
		update_platform_info
	fi

	process_rules "${env}" /etc/velostrata/actions/${phase}.rules "${after}"
}


platform=$(detect_platform)
phase=$(detect_phase ${platform})
after=$1

if [ "${phase}" == "unknown" ] ; then
	velos_log "Unable to detect phase"
elif [ "${phase}" == "hc" ] ; then
	velos_log "HC containerization detected, bypassing all actions"
elif [ "${phase}" == "idle" ] ; then
	velos_log "No change in state detected, bypassing all actions"
else
	match_and_run "${phase}" "${platform}" "${after}"
fi
