#!/bin/sh

# Write directly to console to make sure our output is seen if system is not
# configured properly
console_log() {
  echo "migrate-for-gce: dracut: $*" > /dev/kmsg
}

# In RHEL8/Centos8 "multipath -u" changed to return
# DM_MULTIPATH_DEVICE_PATH=0 when device is not multipath instead of returning
# empty output - but the checks in dracut always checked for non empty path
# instead for the value "1". This was fixed upstream in dracut:
#
#   9dd9329f - modules.d: fix udev rules detection of multipath devices
#
# Apply fix here for OSs without this fix
fix_multipath() {
  for file in /etc/udev/rules.d/*.rules ; do
    sed -i -e \
      's/ENV{DM_MULTIPATH_DEVICE_PATH}=="?\*"/ENV{DM_MULTIPATH_DEVICE_PATH}=="1"/g' \
      "$file"
  done
}

fix_multipath

modprobe iscsi_ibft

if [ ! -e /sys/firmware/ibft ] ; then
  console_log "no iBFT detected - skipping iscsi boot"
  return 0
fi

console_log "iBFT detected - configuring iscsi boot"

{
  # Forces invocation of "ibft_to_cmdline()" by dracut
  # * Rename iface to ibftX
  # * Adds ip=ibft:dhcp to effective cmdline
  # Forces addition of rd.neednet to effective cmdline (existence of ip=
  # lines)
  echo "rd.iscsi.ibft ip=ibft"
  # Forces parse-iscsiroot to invoke iscsiroot (iscsi boot)
  echo "rd.iscsi.firmware iscsi_firmware"
  # Forces multipath
  echo "rd.multipath"
  # Forces retry to secondary iscsi target from ibft if primary is offline -
  # we need more than one retry for that
  echo "iscsi_param=node.session.initial_login_retry_max=4"
  echo "rd.iscsi.param=node.session.initial_login_retry_max=4"
} >> /etc/cmdline
# reread cmdline
unset CMDLINE

# Directly use udev to trigger iscsi-ibft-routes
# By choosing a name that starts with "00-" we ensure that iscsi-ibft-routes
# will run on every interface (except 'lo') before ifup does.
# iscsi-ibft-routes will set the the interface settings (using iscsistart -N)
# or at least set the routes correctly before ifup has a chance to set the rest
# of the settings using dhcp.
# After that, the iscsi session will start without failing.
# This works on all versions of dracut (and opensuse-dracut)
{
  runcmd="/sbin/initqueue --onetime --unique "
  runcmd="${runcmd} --name 00-iscsi-ibft-routes-\$env{INTERFACE} "
  runcmd="${runcmd} /sbin/iscsi-ibft-routes \$env{INTERFACE}"
  rule="SUBSYSTEM==\"net\", "
  rule="${rule} ACTION==\"add\", "
  rule="${rule} ENV{INTERFACE}!=\"lo\", "
  rule="${rule} RUN+=\"${runcmd}\""
  echo "${rule}"
} >> /etc/udev/rules.d/00-migrate-for-gce-hook.rules

# In RHEL7/Centos7 dracut creates the following file in the pre-pivot stage on
# the real root (/sysroot):
#
#   /etc/sysconfig/network-scripts/ifcfg-ibft0
#
# This file contains HWADDR={mac address of ibft0 interface}
# Later on, when the system boots, udev gets loaded and starts processing its
# rules.
# One of them is /usr/lib/udev/rules.d/60-net.rules which invokes the
# /lib/udev/rename_device helper utility. This helper utility
# iterates on all /etc/sysconfig/network-scripts/ifcfg-*suffix* files - if a
# file contains HWADDR with a mac address of an existing interface - it renames
# this interface to the given name in the ifcfg file.
# After detach the interface becomes eth0 and later renamed to ibft0 since the
# mac address of the interface does not change.
#
# Avoid this behavior by removing the write-ifcfg hook in dracut which generates
# the /etc/sysconfig/network-scripts/ifcfg-ibft0 file in the first place.
for hook in /lib/dracut/hooks/pre-pivot/*-write-ifcfg.sh ; do
  if [ ! -f "${hook}" ] ; then
    continue
  fi
  console_log "Deleting ${hook}"
  rm "${hook}"
done